// ignore_for_file: avoid_classes_with_only_static_members

import 'dart:convert';
import 'dart:math';

import 'package:flutter/foundation.dart';
import 'package:http/http.dart';

import 'package:sghi_core/communities/core/endpoints.dart';
import 'package:sghi_core/communities/core/utils.dart';
import 'package:sghi_core/communities/models/user.dart';
import 'package:sghi_core/flutter_graphql_client/constants.dart';
import 'package:sghi_core/flutter_graphql_client/i_flutter_graphql_client.dart';

class ChatAPI {
  static Future<Map<String, dynamic>> createRoom({
    required String roomName,
    required String topic,
    required IGraphQlClient client,
  }) async {
    final String alias = roomName.replaceAll(' ', '');

    final Map<String, dynamic> variables = <String, dynamic>{
      'visibility': 'private',
      'is_direct': false,
      'preset': 'private_chat',
      'invite': <String>[
        // Add a list of users to invite to this group
      ],
      'name': roomName,
      'room_alias_name': alias,
      'topic': topic,
    };

    final Response response = await client.callRESTAPI(
      endpoint: createRoomEndpoint,
      method: kPostMethodName,
      variables: variables,
    );

    final Map<String, dynamic> decodedResponse =
        json.decode(response.body) as Map<String, dynamic>;

    return decodedResponse;
  }

  static Future<Response> deleteMessage({
    required String roomID,
    required String eventID,
    required String senderID,
    required String userID,
    required IGraphQlClient client,
  }) async {
    final String tempTxnID = Random.secure().nextInt(1 << 32).toString();

    final Response response = await client.callRESTAPI(
      endpoint: deleteMessageEventEndpoint(roomID, tempTxnID, eventID),
      method: kPutMethodName,
      variables: <String, dynamic>{},
    );

    return response;
  }

  static Future<Uint8List?> fetchMedia({
    required String? mediaUri,
    required IGraphQlClient client,
  }) async {
    final List<String?>? mediaUriParts = mediaUri?.split('/');
    final String? mediaID = mediaUriParts?.last;

    if (mediaID?.isNotEmpty ?? false) {
      final String endpoint = fetchMediaEndpoint(mediaID!);
      final Response response = await client.callRESTAPI(
        endpoint: endpoint,
        method: kGetMethodName,
        variables: <String, dynamic>{},
      );
      return response.bodyBytes;
    }
    return null;
  }

  static Future<List<RoomUser>> fetchRoomMembers({
    required String roomID,
    required IGraphQlClient client,
  }) async {
    final String url = joinedMembersEndpoint(roomID);

    final Map<String, dynamic> powerLevels =
        await fetchRoomPowerLevels(roomID, client);

    final Map<String, dynamic>? powerLevelUser =
        powerLevels['users'] as Map<String, dynamic>?;

    final Response response = await client.callRESTAPI(
      endpoint: url,
      method: kGetMethodName,
      variables: <String, dynamic>{},
    );

    final Map<String, dynamic> decodedResponse =
        json.decode(response.body) as Map<String, dynamic>;

    final Map<String, dynamic> members =
        decodedResponse['joined'] as Map<String, dynamic>;

    final Map<dynamic, RoomUser> users =
        Map<dynamic, dynamic>.from(members).map(
      (dynamic key, dynamic value) {
        final Map<String, dynamic> v = value as Map<String, dynamic>;

        int? userLevel = 0;

        if (powerLevelUser?.containsKey(key) ?? false) {
          userLevel = powerLevelUser?[key.toString()] as int;
        }

        return MapEntry<dynamic, RoomUser>(
          key,
          RoomUser(
            powerLevel: userLevel,
            moderator: userLevel >= 50,
            roomUser: User(
              userID: key.toString(),
              avatarUri: v['avatar_url'] as String?,
              displayName: v['display_name'] as String?,
            ),
          ),
        );
      },
    );

    return users.values.toList();
  }

  static Future<Response> inviteUsers({
    required IGraphQlClient client,
    required String roomID,
    required String userID,
  }) async {
    final Response response = await client.callRESTAPI(
      endpoint: inviteUserToRoomEndpoint(roomID),
      method: kPostMethodName,
      variables: <String, dynamic>{'user_id': userID},
    );
    return response;
  }

  static Future<Map<String, dynamic>> joinRoom({
    required IGraphQlClient client,
    required String roomID,
  }) async {
    final Response response = await client.callRESTAPI(
      endpoint: joinRoomEndpoint(roomID),
      method: kPostMethodName,
      variables: <String, dynamic>{},
    );

    final Map<String, dynamic> decodedResponse =
        json.decode(response.body) as Map<String, dynamic>;

    return decodedResponse;
  }

  static Future<Response> kickUser({
    required IGraphQlClient client,
    required String roomID,
    required String userID,
  }) async {
    final Response response = await client.callRESTAPI(
      endpoint: kickUserEndpoint(roomID),
      method: kPostMethodName,
      variables: <String, dynamic>{
        'reason': '',
        'user_id': userID,
      },
    );
    return response;
  }

  static Future<Response> leaveRoom({
    required IGraphQlClient client,
    required String roomID,
  }) async {
    final Response response = await client.callRESTAPI(
      endpoint: leaveRoomEndpoint(roomID),
      method: kPostMethodName,
      variables: <String, dynamic>{},
    );
    return response;
  }

  static Future<Response> searchGroupMembers({
    required IGraphQlClient client,
    required String searchParam,
  }) async {
    final Response response = await client.callRESTAPI(
      endpoint: searchMembersEndpoint,
      method: kPostMethodName,
      variables: <String, dynamic>{'limit': 10, 'search_term': searchParam},
    );
    return response;
  }

  static Future<User> signIn({
    required IGraphQlClient client,
    String? username,
    String? password,
  }) async {
    const String loginAuthType = 'm.login.password';

    final Map<String, dynamic> variables = <String, dynamic>{
      'type': loginAuthType,
      'identifier': <String, dynamic>{'type': 'm.id.user', 'user': username},
      'password': password,
    };

    final Response response = await client.callRESTAPI(
      endpoint: loginEndpoint,
      method: kPostMethodName,
      variables: variables,
    );

    final Map<String, dynamic> decodedResponse =
        json.decode(response.body) as Map<String, dynamic>;

    return User.fromJson(decodedResponse);
  }

  static Future<Map<String, dynamic>> sendMessage({
    required IGraphQlClient client,
    required String roomID,
    required String message,
  }) async {
    final String tempTxnID = Random.secure().nextInt(1 << 32).toString();

    final Response response = await client.callRESTAPI(
      endpoint: sendMessageEventEndpoint(roomID, tempTxnID),
      method: kPutMethodName,
      variables: <String, dynamic>{'body': message, 'msgtype': 'm.text'},
    );

    return json.decode(utf8.decode(response.bodyBytes)) as Map<String, dynamic>;
  }
}
