import 'dart:convert';
import 'dart:typed_data';

import 'package:flutter_test/flutter_test.dart';
import 'package:http/http.dart';
import 'package:sghi_core/communities/core/chat_api.dart';
import 'package:sghi_core/communities/models/user.dart';

import '../../mocks.dart';

void main() {
  group('ChatAPI', () {
    test('create room works correctly', () async {
      final Map<String, dynamic> response = await ChatAPI.createRoom(
        roomName: 'roomName',
        topic: 'topic',
        client: MockShortGraphQlClient.withResponse(
          '',
          '',
          Response(
            jsonEncode(<String, dynamic>{'data': 'some data goes here'}),
            200,
          ),
        ),
      );
      expect(response, isA<Map<String, dynamic>>());
      expect(response['data'], 'some data goes here');
    });

    test('delete message  works correctly', () async {
      final Response response = await ChatAPI.deleteMessage(
        roomID: 'roomID',
        eventID: 'eventID',
        senderID: 'senderID',
        userID: 'userID',
        client: MockShortGraphQlClient.withResponse(
          '',
          '',
          Response(
            jsonEncode(<String, dynamic>{'data': 'some data goes here'}),
            200,
          ),
        ),
      );
      expect(json.decode(response.body)['data'], 'some data goes here');
    });

    test('fetch media works correctly when mediaID is not available', () async {
      final Uint8List? response = await ChatAPI.fetchMedia(
        mediaUri: 'mediaUri',
        client: MockShortGraphQlClient.withResponse(
          '',
          '',
          Response(
            '',
            200,
          ),
        ),
      );
      expect(response, isEmpty);
    });

    test('fetch media works correctly', () async {
      final Response mockedResponse = Response(
        jsonEncode(<String, dynamic>{'data': 'some data goes here'}),
        200,
      );
      final Uint8List bodyBytes = mockedResponse.bodyBytes;

      final Uint8List? response = await ChatAPI.fetchMedia(
        mediaUri: 'mediaUri/id',
        client: MockShortGraphQlClient.withResponse('', '', mockedResponse),
      );
      expect(response, bodyBytes);
    });

    test('invite users  works correctly', () async {
      final Response response = await ChatAPI.inviteUsers(
        roomID: 'roomID',
        userID: 'userID',
        client: MockShortGraphQlClient.withResponse(
          '',
          '',
          Response(
            jsonEncode(<String, dynamic>{'data': 'some data goes here'}),
            200,
          ),
        ),
      );
      expect(json.decode(response.body)['data'], 'some data goes here');
    });

    test('join room works correctly', () async {
      final Map<String, dynamic> response = await ChatAPI.joinRoom(
        roomID: 'roomID',
        client: MockShortGraphQlClient.withResponse(
          '',
          '',
          Response(
            jsonEncode(<String, dynamic>{'data': 'some data goes here'}),
            200,
          ),
        ),
      );
      expect(response['data'], 'some data goes here');
    });

    test('kickUser works correctly', () async {
      final Response response = await ChatAPI.kickUser(
        roomID: 'roomID',
        userID: 'userID',
        client: MockShortGraphQlClient.withResponse(
          '',
          '',
          Response(
            jsonEncode(<String, dynamic>{'data': 'some data goes here'}),
            200,
          ),
        ),
      );
      expect(json.decode(response.body)['data'], 'some data goes here');
    });

    test('leaveRoom works correctly', () async {
      final Response response = await ChatAPI.leaveRoom(
        roomID: 'roomID',
        client: MockShortGraphQlClient.withResponse(
          '',
          '',
          Response(
            jsonEncode(<String, dynamic>{'data': 'some data goes here'}),
            200,
          ),
        ),
      );
      expect(json.decode(response.body)['data'], 'some data goes here');
    });

    test('searchGroupMembers works correctly', () async {
      final Response response = await ChatAPI.searchGroupMembers(
        searchParam: 'searchParam',
        client: MockShortGraphQlClient.withResponse(
          '',
          '',
          Response(
            jsonEncode(<String, dynamic>{'data': 'some data goes here'}),
            200,
          ),
        ),
      );
      expect(json.decode(response.body)['data'], 'some data goes here');
    });
    test('fetchRoomMembers works correctly', () async {
      final List<RoomUser> users = await ChatAPI.fetchRoomMembers(
          roomID: 'roomID', client: MockGraphQlClient());
      expect(users.length, 1);
    });
    test('signIn works correctly', () async {
      final User user = await ChatAPI.signIn(
        client: MockGraphQlClient(),
      );

      expect(user.displayName, 'Abiud Orina');
    });
    test('sendMessage works correctly', () async {
      final Map<String, dynamic> response = await ChatAPI.sendMessage(
        roomID: 'roomID',
        message: 'testMessage',
        client: MockShortGraphQlClient.withResponse(
          '',
          '',
          Response(
            jsonEncode(<String, dynamic>{'data': 'some data goes here'}),
            200,
          ),
        ),
      );
      expect(response['data'], 'some data goes here');
    });
  });
}
